import os
import subprocess

from charmhelpers.core import host
from charmhelpers.core import hookenv
from charmhelpers.core import services
from charmhelpers.contrib.cloudfoundry import contexts


__all__ = ['CF_DIR', 'CC_PACKAGES', 'CC_DIR', 'CC_CONFIG_DIR',
           'CC_CONFIG_FILE', 'CC_DB_FILE', 'CC_JOB_FILE',
           'CC_LOG_DIR', 'CC_RUN_DIR',
           'NGINX_JOB_FILE', 'NGINX_CONFIG_FILE',
           'NGINX_RUN_DIR', 'NGINX_LOG_DIR', 'FOG_CONNECTION']

CC_PACKAGES = ['cfcloudcontroller', 'runit', 'golang-go', 'python-jinja2']
CF_DIR = '/var/lib/cloudfoundry'
CC_DIR = os.path.join(CF_DIR, 'cfcloudcontroller')
CC_CONFIG_DIR = os.path.join(CC_DIR, 'config')
# this one goes from package
CC_CONFIG_FILE = os.path.join(CC_CONFIG_DIR, 'cloud_controller.yml')
CC_DB_FILE = os.path.join(CC_DIR, 'db/cc.db')
CC_JOB_FILE = '/etc/init/cf-cloudcontroller.conf'
CC_LOG_DIR = '/var/vcap/sys/log/cloud_controller_ng'
CC_RUN_DIR = '/var/vcap/sys/run/cloud_controller_ng'

NGINX_JOB_FILE = '/etc/init/cf-nginx.conf'
NGINX_CONFIG_FILE = os.path.join(CC_CONFIG_DIR, 'nginx.conf')
NGINX_RUN_DIR = '/var/vcap/sys/run/nginx_ccng'
NGINX_LOG_DIR = '/var/vcap/sys/log/nginx_ccng'
FOG_CONNECTION = '/var/vcap/nfs/store'


def db_migrate(service_name):
    hookenv.log("Starting db:migrate...", hookenv.DEBUG)
    with host.chdir(CC_DIR):
        subprocess.check_call([
            'sudo', '-u', 'vcap', '-g', 'vcap',
            'CLOUD_CONTROLLER_NG_CONFIG={}'.format(CC_CONFIG_FILE),
            'bundle', 'exec', 'rake', 'db:migrate'])
    hookenv.log("Finished db:migrate", hookenv.DEBUG)


API_CONTEXT = contexts.StoredContext('api_credentials.yml', {
    'api': {
        'hostname': hookenv.unit_get('private-address').encode('utf-8'),
        'port': 9022,
        'user': host.pwgen(7),
        'password': host.pwgen(7),
    }})


SERVICES = [
    {
        'service': 'cf-cloudcontroller',
        'required_data': [contexts.NatsRelation(),
                          contexts.RouterRelation(),
                          contexts.MysqlRelation(),
                          API_CONTEXT],
        'data_ready': [
            services.template(source='cf-cloudcontroller.conf',
                              target='/etc/init/cf-cloudcontroller.conf'),
            services.template(source='cloud_controller.yml',
                              target=CC_CONFIG_FILE,
                              owner='vcap'),
            db_migrate,
        ],
    },
    {
        'service': 'cf-cloudcontroller-job',
        'data_ready': services.template(
            source='cf-cloudcontroller-job.conf',
            target='/etc/init/cf-cloudcontroller-job.conf'),
    },
]
